<p><?php print_string('welcomep30', 'install', $distro) ?></p>
<ul>
  <li><?php p($distro->webname) ?> <?php p($distro->webversion) ?></li>
  <li><?php p($distro->phpname) ?> <?php p($distro->phpversion) ?></li>
  <li><?php p($distro->dbname) ?> <?php p($distro->dbversion) ?></li>
</ul>
<p><?php print_string('welcomep40', 'install', $distro) ?></p>
<p><?php print_string('welcomep50', 'install', $distro) ?></p>
<p><?php print_string('welcomep60', 'install', $distro) ?></p>
<p><?php print_string('welcomep70', 'install', $distro) ?></p>
