<table width="100%" cellpadding="5">
    <tr>
        <td class="c0"><label for="name"><?php echo get_string('fieldname', 'data'); ?></label></td>
        <td class="c1"><input class="fieldname" type="text" name="name" id="name" value="<?php p($this->field->name); ?>" /></td>
    </tr>
    <tr>
        <td class="c0"><label for="description"><?php echo get_string('fielddescription', 'data'); ?></label></td>
        <td class="c1">
            <input class="fielddescription" type="text" name="description" id="description"
            value="<?php p($this->field->description); ?>" />
        </td>
    </tr>
    <tr>
        <td class="c0"><label for="required"><?php echo get_string('requiredfield', 'data'); ?></label></td>
        <td class="c1"><input class="requiredfield" type="checkbox" name="required" id="required" <?php p($this->field->required?"checked=\"checked\"":""); ?>/></td>
    </tr>
    <tr>
        <td class="c0"><label for="param2">
            <?php echo get_string('fieldwidth', 'data'); ?></label></td>
        <td class="c1">
            <input class="textareafieldsize" type="text" name="param2" id="param2" value=
            <?php
                if (empty($this->field->param2)) {
                    echo('"60"');
                } else {
                    echo '"'.s($this->field->param2).'"';
                }
            ?> /> <?php echo get_string('columns', 'data'); ?>
        </td>
    </tr>
    <tr>
        <td class="c0"><label for="param3">
            <?php echo get_string('fieldheight', 'data'); ?></label></td>
        <td class="c1">
            <input class="textareafieldsize" type="text" name="param3" id="param3" value=
            <?php
                if (empty($this->field->param3)) {
                    echo('"35"');
                } else {
                    echo '"'.s($this->field->param3).'"';
                }
            ?> /> <?php echo get_string('rows', 'data'); ?>
        </td>
    </tr>
    <tr>
        <td class="c0"><label for="param5"><?php echo get_string('maxbytes', 'datafield_textarea'); ?></label></td>
        <td class="c1">
            <input class="c1" type="text" name="param5" id="param5"
                   value="<?php if (isset($this->field->param5)) { p($this->field->param5); } else { echo '0'; } ?>" />
            <?php echo get_string('maxbytes_desc', 'datafield_textarea'); ?>
        </td>
    </tr>
</table>
<input type="hidden" name="param4" id="param4" value="<?php echo FORMAT_HTML; ?>" />
