<table class="main" cellpadding="10" cellspacing="0">
  <tbody>
    <tr>
      <td class="td_mainheading" colspan="2">
        <p class="p_mainheading"><strong><?php print_string('welcomep10', 'install', $a) ?></strong></p>
        <p><?php print_string('welcomep20', 'install', $a) ?></p>
        <p><?php print_string('welcomep30', 'install', $a) ?></p>
        <ul>
          <li><?php p($a->webname) ?> <?php p($a->webversion) ?></li>
          <li><?php p($a->phpname) ?> <?php p($a->phpversion) ?></li>
          <li><?php p($a->dbname) ?> <?php p($a->dbversion) ?></li>
        </ul>
        <p><?php print_string('welcomep40', 'install', $a) ?></p>
        <p><?php print_string('welcomep50', 'install', $a) ?></p>
        <p><?php print_string('welcomep60', 'install', $a) ?></p>
        <p><?php print_string('welcomep70', 'install', $a) ?></p>
      </td>
    </tr>
  </tbody>
</table>
