<?php

/**
 * Licensed to Jasig under one or more contributor license
 * agreements. See the NOTICE file distributed with this work for
 * additional information regarding copyright ownership.
 *
 * Jasig licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * PHP Version 7
 *
 * @file     CAS/ServerHostname/Interface.php
 * @category Authentication
 * @package  PhpCAS
 * @author   Henry Pan <git@phy25.com>
 * @license  http://www.apache.org/licenses/LICENSE-2.0  Apache License 2.0
 * @link     https://wiki.jasig.org/display/CASC/phpCAS
 */

/**
 * An interface for classes that gets the server name of the PHP server.
 * This is used to generate service URL and PGT callback URL.
 *
 * @class    CAS_ServiceBaseUrl_Interface
 * @category Authentication
 * @package  PhpCAS
 * @author   Henry Pan <git@phy25.com>
 * @license  http://www.apache.org/licenses/LICENSE-2.0  Apache License 2.0
 * @link     https://wiki.jasig.org/display/CASC/phpCAS
 */
interface CAS_ServiceBaseUrl_Interface
{

    /**
     * Get PHP HTTP protocol and server name.
     *
     * @return string protocol, server hostname, and optionally port,
     *                without trailing slash (https://localhost:8443)
     */
    public function get();

    /**
     * Check whether HTTPS is used.
     *
     * This is used to construct the protocol in the URL.
     *
     * @return bool true if HTTPS is used
     */
    public function isHttps();

}
