<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Automatically generated strings for Moodle installer
 *
 * Do not edit this file manually! It contains just a subset of strings
 * needed during the very first steps of installation. This file was
 * generated automatically by export-installer.php (which is part of AMOS
 * {@link http://docs.moodle.org/dev/Languages/AMOS}) using the
 * list of strings defined in /install/stringnames.txt.
 *
 * @package   installer
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

$string['cannotcreatelangdir'] = 'Nun pue crear un direutoriu d\'idioma.';
$string['cannotcreatetempdir'] = 'Nun pue crear un direutoriu de tiempu.';
$string['cannotdownloadcomponents'] = 'Nun pue descargar componentes.';
$string['cannotdownloadzipfile'] = 'Nun pue descargar ficheros ZIP.';
$string['cannotfindcomponent'] = 'Nun puen atopaser componentes.';
$string['cannotsavemd5file'] = 'Nun se pue guardar el ficheru md5.';
$string['cannotsavezipfile'] = 'Nun se pue guardar un ficheru ZIP.';
$string['cannotunzipfile'] = 'Nun se pue descomprimir el ficheru.';
$string['componentisuptodate'] = 'El compoente ta actualizáu.';
$string['downloadedfilecheckfailed'] = 'Falló la verificación del ficheru descargáu.';
$string['invalidmd5'] = 'md5 inválidu.';
$string['missingrequiredfield'] = 'Falta dalgún campu obligatoriu.';
$string['remotedownloaderror'] = 'La descarga del componente al so servidor falló. Por favor, asegúrese de que les carauterístiques del proxy son les correutes. La estensión PHP cURL ye perrecomendable.<br/><br/>Tien de descargar el ficheru <a href="{$a->url}">{$a->url}</a> manualmente, copialu en "{$a->dest}" del so servidor y descomprimilu ellí.';
$string['wrongdestpath'] = 'Ruta de destín equivocada.';
$string['wrongsourcebase'] = 'URL de la fonte equivocada.';
$string['wrongzipfilename'] = 'Nome de ficheru ZIP erroneu.';
