<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'survey', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package   mod_survey
 * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['actual'] = 'Actual';
$string['actualclass'] = 'Class actual';
$string['actualstudent'] = '{$a} actual';
$string['allquestions'] = 'All questions in order, all students';
$string['allscales'] = 'All scales, all students';
$string['alreadysubmitted'] = 'You have already submitted this survey';
$string['analysisof'] = 'Analysis of {$a}';
$string['answers'] = 'Answers';
$string['attlsintro'] = 'The purpose of this questionnaire is to help us evaluate your attitudes towards thinking and learning.

There are no \'right\' or \'wrong\' answers; we are interested only in your opinion. Please be assured that your responses will be treated with a high degree of confidentiality, and will not affect your assessment.';
$string['attlsmintro'] = 'In discussion ...';
$string['attlsm1'] = 'Attitudes Towards Thinking and Learning';
$string['attlsm2'] = 'Connected learning';
$string['attlsm3'] = 'Separate learning';
$string['attlsname'] = 'ATTLS (20 item version)';
$string['attls1'] = 'In evaluating what someone says, I focus on the quality of their argument, not on the person who\'s presenting it.';
$string['attls1short'] = 'focus quality of argument';
$string['attls10'] = 'It\'s important for me to remain as objective as possible when I analyze something.';
$string['attls10short'] = 'remain objective';
$string['attls11'] = 'I try to think with people instead of against them.';
$string['attls11short'] = 'think WITH people';
$string['attls12'] = 'I have certain criteria I use in evaluating arguments.';
$string['attls12short'] = 'use criteria to evaluate';
$string['attls13'] = 'I\'m more likely to try to understand someone else\'s opinion than to try to evaluate it.';
$string['attls13short'] = 'try to understand';
$string['attls14'] = 'I try to point out weaknesses in other people\'s thinking to help them clarify their arguments.';
$string['attls14short'] = 'point out weaknesses';
$string['attls15'] = 'I tend to put myself in other people\'s shoes when discussing controversial issues, to see why they think the way they do.';
$string['attls15short'] = 'put myself in their shoes';
$string['attls16'] = 'One could call my way of analysing things \'putting them on trial\' because I am careful to consider all the evidence.';
$string['attls16short'] = 'putting on trial';
$string['attls17'] = 'I value the use of logic and reason over the incorporation of my own concerns when solving problems.';
$string['attls17short'] = 'I value logic most';
$string['attls18'] = 'I can obtain insight into opinions that differ from mine through empathy.';
$string['attls18short'] = 'insight from empathy';
$string['attls19'] = 'When I encounter people whose opinions seem alien to me, I make a deliberate effort to \'extend\' myself into that person, to try to see how they could have those opinions.';
$string['attls19short'] = 'make effort to extend';
$string['attls2'] = 'I like playing devil\'s advocate - arguing the opposite of what someone is saying.';
$string['attls2short'] = 'play devil\'s advocate';
$string['attls20'] = 'I spend time figuring out what\'s \'wrong\' with things. For example, I\'ll look for something in a literary interpretation that isn\'t argued well enough.';
$string['attls20short'] = 'what\'s wrong?';
$string['attls3'] = 'I like to understand where other people are \'coming from\', what experiences have led them to feel the way they do.';
$string['attls3short'] = 'where people come from';
$string['attls4'] = 'The most important part of my education has been learning to understand people who are very different to me.';
$string['attls4short'] = 'understand different people';
$string['attls5'] = 'I feel that the best way for me to achieve my own identity is to interact with a variety of other people.';
$string['attls5short'] = 'interact with variety';
$string['attls6'] = 'I enjoy hearing the opinions of people who come from backgrounds different to mine - it helps me to understand how the same things can be seen in such different ways.';
$string['attls6short'] = 'enjoy hearing opinions';
$string['attls7'] = 'I find that I can strengthen my own position through arguing with someone who disagrees with me.';
$string['attls7short'] = 'strengthen by argue';
$string['attls8'] = 'I am always interested in knowing why people say and believe the things they do.';
$string['attls8short'] = 'know why people do';
$string['attls9'] = 'I often find myself arguing with the authors of books that I read, trying to logically figure out why they\'re wrong.';
$string['attls9short'] = 'argue with authors';
$string['cannotfindanswer'] = 'There are no answers for this survey yet.';
$string['cannotfindquestion'] = 'Question doesn\'t exist';
$string['cannotfindsurveytmpt'] = 'No survey templates found!';
$string['completiondetail:submit'] = 'Submit answers';
$string['completionsubmit'] = 'Student must make a submission';
$string['ciqintro'] = 'While thinking about recent events in this class, answer the questions below.';
$string['ciqname'] = 'Critical incidents';
$string['ciq1'] = 'At what moment in class were you most engaged as a learner?';
$string['ciq1short'] = 'Most engaged';
$string['ciq2'] = 'At what moment in class were you most distanced as a learner?';
$string['ciq2short'] = 'Most distanced';
$string['ciq3'] = 'What action from anyone in the forums did you find most affirming or helpful?';
$string['ciq3short'] = 'Helpful moment';
$string['ciq4'] = 'What action from anyone in the forums did you find most puzzling or confusing?';
$string['ciq4short'] = 'Confusing moment';
$string['ciq5'] = 'What event surprised you most?';
$string['ciq5short'] = 'Suprising moment';
$string['clicktocontinue'] = 'Click here to continue';
$string['clicktocontinuecheck'] = 'Click here to check and continue';
$string['collesaintro'] = 'The purpose of this survey is to help us understand how well the online delivery of this unit enabled you to learn.

Each one of the 24 statements below asks about your experience in this unit.

There are no \'right\' or \'wrong\' answers; we are interested only in your opinion. Please be assured that your responses will be treated with a high degree of confidentiality, and will not affect your assessment.

Your carefully considered responses will help us improve the way this unit is presented online in the future.

Thanks very much.';
$string['collesaname'] = 'COLLES (Actual)';
$string['collesapintro'] = 'The purpose of this questionnaire is to help us understand how well the online delivery of this unit enabled you to learn.

Each couple of the 24 statements below asks you to compare your <b>preferred</b> (ideal) and <b>actual</b> experience in this unit.

There are no \'right\' or \'wrong\' answers; we are interested only in your opinion. Please be assured that your responses will be treated with a high degree of confidentiality, and will not affect your assessment.

Your carefully considered responses will help us improve the way this unit is presented online in the future.

Thanks very much.';
$string['collesapname'] = 'COLLES (Preferred and Actual)';
$string['collesmintro'] = 'In this online unit...';
$string['collesm1'] = 'Relevance';
$string['collesm1short'] = 'Relevance';
$string['collesm2'] = 'Reflective thinking';
$string['collesm2short'] = 'Reflective thinking';
$string['collesm3'] = 'Interactivity';
$string['collesm3short'] = 'Interactivity';
$string['collesm4'] = 'Tutor support';
$string['collesm4short'] = 'Tutor support';
$string['collesm5'] = 'Peer support';
$string['collesm5short'] = 'Peer support';
$string['collesm6'] = 'Interpretation';
$string['collesm6short'] = 'Interpretation';
$string['collespintro'] = 'The purpose of this survey is to help us understand what you value in an online learning experience.

Each one of the 24 statements below asks about your <b>preferred</b> (ideal) experience in this unit.

There are no \'right\' or \'wrong\' answers; we are interested only in your opinion. Please be assured that your responses will be treated with a high degree of confidentiality, and will not affect your assessment.

Your carefully considered responses will help us improve the way this unit is presented online in the future.

Thanks very much.';
$string['collespname'] = 'COLLES (Preferred)';
$string['colles1'] = 'my learning focuses on issues that interest me.';
$string['colles1short'] = 'focus on interesting issues';
$string['colles10'] = 'I ask other students to explain their ideas.';
$string['colles10short'] = 'I ask for explanations';
$string['colles11'] = 'other students ask me to explain my ideas.';
$string['colles11short'] = 'I\'m asked to explain';
$string['colles12'] = 'other students respond to my ideas.';
$string['colles12short'] = 'students respond to me';
$string['colles13'] = 'the tutor stimulates my thinking.';
$string['colles13short'] = 'tutor stimulates thinking';
$string['colles14'] = 'the tutor encourages me to participate.';
$string['colles14short'] = 'tutor encourages me';
$string['colles15'] = 'the tutor models good discourse.';
$string['colles15short'] = 'tutor models discourse';
$string['colles16'] = 'the tutor models critical self-reflection.';
$string['colles16short'] = 'tutor models self-reflection';
$string['colles17'] = 'other students encourage my participation.';
$string['colles17short'] = 'students encourage me';
$string['colles18'] = 'other students praise my contribution.';
$string['colles18short'] = 'students praise me';
$string['colles19'] = 'other students value my contribution.';
$string['colles19short'] = 'students value me';
$string['colles2'] = 'what I learn is important for my professional practice.';
$string['colles2short'] = 'important to my practice';
$string['colles20'] = 'other students empathise with my struggle to learn.';
$string['colles20short'] = 'students empathise';
$string['colles21'] = 'I make good sense of other students\' messages.';
$string['colles21short'] = 'I understand other students';
$string['colles22'] = 'other students make good sense of my messages.';
$string['colles22short'] = 'students understand me';
$string['colles23'] = 'I make good sense of the tutor\'s messages.';
$string['colles23short'] = 'I understand the tutor';
$string['colles24'] = 'the tutor makes good sense of my messages.';
$string['colles24short'] = 'tutor understands me';
$string['colles3'] = 'I learn how to improve my professional practice.';
$string['colles3short'] = 'improve my practice';
$string['colles4'] = 'what I learn connects well with my professional practice.';
$string['colles4short'] = 'connects with my practice';
$string['colles5'] = 'I think critically about how I learn.';
$string['colles5short'] = 'I\'m critical of my learning';
$string['colles6'] = 'I think critically about my own ideas.';
$string['colles6short'] = 'I\'m critical of my own ideas';
$string['colles7'] = 'I think critically about other students\' ideas.';
$string['colles7short'] = 'I\'m critical of other students';
$string['colles8'] = 'I think critically about ideas in the readings.';
$string['colles8short'] = 'I\'m critical of readings';
$string['colles9'] = 'I explain my ideas to other students.';
$string['colles9short'] = 'I explain my ideas';
$string['customintro'] = 'Description';
$string['deleteallanswers'] = 'Delete all survey responses';
$string['deleteanalysis'] = 'Delete response analysis';
$string['done'] = 'Done';
$string['download'] = 'Download';
$string['downloadexcel'] = 'Download data as Excel spreadsheet';
$string['downloadinfo'] = 'You can download the complete raw data for this survey in a form suitable for analysis in Excel, SPSS or other package.';
$string['downloadresults'] = 'Download results';
$string['downloadtext'] = 'Download data as a plain text file';
$string['editingasurvey'] = 'Editing a survey';
$string['errorunabletosavenotes'] = 'An error occurred while saving your notes.';
$string['eventreportdownloaded'] = 'Survey report downloaded';
$string['eventreportviewed'] = 'Survey report viewed';
$string['eventresponsesubmitted'] = 'Survey response submitted';
$string['guestsnotallowed'] = 'Only enrolled users are able to submit surveys';
$string['howlong'] = 'How long did this survey take you to complete?';
$string['howlongoptions'] = 'under 1 min,1-2 min,2-3 min,3-4 min,4-5-min,5-10 min,more than 10';
$string['ifoundthat'] = 'I found that';
$string['indicator:cognitivedepth'] = 'Survey cognitive';
$string['indicator:cognitivedepth_help'] = 'This indicator is based on the cognitive depth reached by the student in a Survey activity.';
$string['indicator:cognitivedepthdef'] = 'Survey cognitive';
$string['indicator:cognitivedepthdef_help'] = 'The participant has reached this percentage of the cognitive engagement offered by the Survey activities during this analysis interval (Levels = No view, View, Submit)';
$string['indicator:cognitivedepthdef_link'] = 'Learning_analytics_indicators#Cognitive_depth';
$string['indicator:socialbreadth'] = 'Survey social';
$string['indicator:socialbreadth_help'] = 'This indicator is based on the social breadth reached by the student in a Survey activity.';
$string['indicator:socialbreadthdef'] = 'Survey social';
$string['indicator:socialbreadthdef_help'] = 'The participant has reached this percentage of the social engagement offered by the Survey activities during this analysis interval (Levels = No participation, Participant alone)';
$string['indicator:socialbreadthdef_link'] = 'Learning_analytics_indicators#Social_breadth';
$string['introtext'] = 'Introduction text';
$string['invalidsurveyid'] = 'Survey ID was incorrect';
$string['invalidtmptid'] = 'Invalid template id';
$string['ipreferthat'] = 'I prefer that';
$string['modulename'] = 'Survey';
$string['modulename_help'] = 'The survey activity module provides a number of verified survey instruments that have been found useful in assessing and stimulating learning in online environments. A teacher can use these to gather data from their students that will help them learn about their class and reflect on their own teaching.

Note that these survey tools are pre-populated with questions. Teachers who wish to create their own survey should use the feedback activity module.';
$string['modulename_link'] = 'mod/survey/view';
$string['modulenameplural'] = 'Surveys';
$string['name'] = 'Name';
$string['newsurveyresponses'] = 'New survey responses';
$string['nobodyyet'] = 'Nobody has yet completed this survey';
$string['notdone'] = 'Not done yet';
$string['notes'] = 'Your private analysis and notes';
$string['othercomments'] = 'Do you have any other comments?';
$string['page-mod-survey-x'] = 'Any survey module page';
$string['peoplecompleted'] = '{$a} people have completed this survey so far';
$string['pluginadministration'] = 'Survey administration';
$string['pluginname'] = 'Survey';
$string['preferred'] = 'Preferred';
$string['preferredclass'] = 'Class preferred';
$string['preferredstudent'] = '{$a} preferred';
$string['privacy:metadata:analysis'] = 'A record of survey answers analysis.';
$string['privacy:metadata:analysis:notes'] = 'Notes saved against a user\'s answers.';
$string['privacy:metadata:analysis:userid'] = 'The ID of the user answering the survey.';
$string['privacy:metadata:answers'] = 'A collection of answers to surveys.';
$string['privacy:metadata:answers:answer1'] = 'Field to store the answer to a question.';
$string['privacy:metadata:answers:answer2'] = 'Additional field to store the answer to a question.';
$string['privacy:metadata:answers:question'] = 'The question.';
$string['privacy:metadata:answers:time'] = 'The time when the answer was posted.';
$string['privacy:metadata:answers:userid'] = 'The ID of the user who submitted their answer.';
$string['question'] = 'Question';
$string['questions'] = 'Questions';
$string['questionsnotanswered'] = 'Some of the multiple choice questions have not been answered.';
$string['report'] = 'Survey report';
$string['responsereports'] = 'Response reports';
$string['responses'] = 'Responses';
$string['savednotes'] = 'Your notes were saved';
$string['scaleagree5'] = 'Strongly disagree,Somewhat disagree,Neither agree nor disagree,Somewhat agree,Strongly agree';
$string['scales'] = 'Scales';
$string['scaletimes5'] = 'Almost never,Seldom,Sometimes,Often,Almost always';
$string['search:activity'] = 'Survey - activity information';
$string['seemoredetail'] = 'Click here to see more detail';
$string['selectedquestions'] = 'Selected questions from a scale, all students';
$string['summary'] = 'Summary';
$string['survey:addinstance'] = 'Add a new survey';
$string['surveycompleted'] = 'You\'ve completed this survey.  The graph below shows a summary of your results compared to the class averages.';
$string['surveycompletednograph'] = 'You have completed this survey.';
$string['survey:download'] = 'Download responses';
$string['surveygraph'] = 'Survey graph';
$string['surveyname'] = 'Survey name';
$string['survey:participate'] = 'Respond to survey';
$string['survey:readresponses'] = 'View responses';
$string['surveysaved'] = 'Survey saved';
$string['surveytype'] = 'Survey type';
$string['surveytype_help'] = 'There are 3 available survey types:

* Attitudes to Thinking and Learning Survey (ATTLS) - For measuring the extent to which a person is a \'connected knower\' (tends to find learning more enjoyable, and is often more cooperative, congenial and more willing to build on the ideas of others) or a \'separate knower\' (tends to take a more critical and argumentative stance to learning)
* Critical incidents survey
* Constructivist On-line Learning Environment Survey (COLLES) - For monitoring the extent to which the interactive capacity of the World Wide Web may be exploited for engaging students in dynamic learning practices';
$string['surveytype_link'] = 'mod/survey/mod';
$string['thanksforanswers'] = 'Thanks for answering this survey, {$a}';
$string['time'] = 'Time';
$string['viewsurveyresponses'] = 'View {$a} survey responses';
$string['notyetanswered'] = 'Not yet answered';
$string['allquestionrequireanswer'] = 'All questions are required and must be answered.';
