<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Automatically generated strings for Moodle installer
 *
 * Do not edit this file manually! It contains just a subset of strings
 * needed during the very first steps of installation. This file was
 * generated automatically by export-installer.php (which is part of AMOS
 * {@link http://docs.moodle.org/dev/Languages/AMOS}) using the
 * list of strings defined in /install/stringnames.txt.
 *
 * @package   installer
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

$string['cannotcreatedboninstall'] = '<p> డేటాబేస్ను సృష్టించలేరు. </ p>
<p> పేర్కొన్న డేటాబేస్ ఉనికిలో లేదు మరియు ఇచ్చిన యూజర్కు డాటాబేస్ను సృష్టించడానికి అనుమతి లేదు. </ p>
<p> సైట్ నిర్వాహకుడు డేటాబేస్ కాన్ఫిగరేషన్ను ధృవీకరించాలి. </ p>';
$string['cannotcreatelangdir'] = 'భాష డైరెక్టరీని సృష్టించలేం';
$string['cannotcreatetempdir'] = 'టెంప్ డైరెక్టరీని సృష్టించలేం';
$string['cannotdownloadcomponents'] = 'కాంపొనెంట్లను దించుకోలేరు';
$string['cannotdownloadzipfile'] = 'ZIP దస్త్రాలను దించుకోలేరు';
$string['cannotfindcomponent'] = 'కాంపొనెంటు కనబడలేదు';
$string['cannotsavemd5file'] = 'Md5 ఫైల్ను సేవ్ చేయలేరు';
$string['cannotsavezipfile'] = 'జిప్ ఫైల్ను సేవ్ చేయడం సాధ్యపడదు';
$string['cannotunzipfile'] = 'ఫైల్ అన్జిప్ చేయలేరు';
$string['componentisuptodate'] = 'కాంపొనెంటు తాజాగా ఉంది';
$string['dmlexceptiononinstall'] = '<p>A డేటాబేస్ లోపం ఏర్పడింది [{$ a-> errorcode}]. <br /> {$a->debuginfo} </p>';
$string['downloadedfilecheckfailed'] = 'డౌన్లోడ్ చేసిన ఫైల్ తనిఖీ విఫలమైంది';
$string['invalidmd5'] = 'చెక్ వేరియబుల్ తప్పు. - మళ్ళీ ప్రయత్నించండి';
$string['missingrequiredfield'] = 'కొన్ని అవసరమైన ఫీల్డ్ లేదు';
$string['remotedownloaderror'] = '<p> మీ సర్వర్ కు భాగం యొక్క డౌన్లోడ్ విఫలమైంది. దయచేసి ప్రాక్సీ సెట్టింగ్లను ధృవీకరించండి; PHP కర్ల్ పొడిగింపు సిఫారసు చేయబడుతుంది.</p>
<p> మీరు <a href="{$a->url} "> {$a->url} </a> ను మానవీయంగా డౌన్ లోడ్ చేసుకోవాలి, దీన్ని మీ సర్వర్లో" {$a->dest} "కు కాపీ చెయ్యండి అక్కడ దాన్ని అన్జిప్ చేయండి.</p>';
$string['wrongdestpath'] = 'తప్పు గమ్యం మార్గం';
$string['wrongsourcebase'] = 'తప్పు మూలం URL బేస్';
$string['wrongzipfilename'] = 'తప్పు జిప్ ఫైల్ పేరు';
