<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'repository_flickr_public', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package   repository_flickr_public
 * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['all'] = 'All';
$string['apikey'] = 'API key';
$string['backtosearch'] = 'Back to search screen';
$string['by'] = 'Attribution license';
$string['by-nc'] = 'Attribution-NonCommercial license';
$string['by-nc-nd'] = 'Attribution-NonCommercial-NoDerivs license';
$string['by-nc-sa'] = 'Attribution-NonCommercial-ShareAlike license';
$string['by-nd'] = 'Attribution-NoDerivs license';
$string['by-sa'] = 'Attribution-ShareAlike license';
$string['callbackurl'] = 'Callback URL';
$string['commercialuse'] = 'I want to use the images commercially';
$string['configplugin'] = 'Flickr public configuration';
$string['creativecommonscommercial'] = 'Only creative commons commercial';
$string['emailaddress'] = 'Email address';
$string['flickr_public:view'] = 'Use Flickr public repository in file picker';
$string['fulltext'] = 'Full text';
$string['information'] = '<div>Get a <a href="http://www.flickr.com/services/api/keys/">Flickr API Key</a> for your Moodle site. </div>';
$string['invalidemail'] = 'Invalid email account for flickr';
$string['license'] = 'Licence';
$string['modification'] = 'I want to be able to modify the images';
$string['notitle'] = 'notitle';
$string['nullphotolist'] = 'There are no photos in this account';
$string['remember'] = 'Remember me';
$string['pluginname_help'] = 'Repository on flickr.com';
$string['pluginname'] = 'Flickr public';
$string['secret'] = 'Secret';
$string['sharealike'] = 'Yes, as long as others share alike';
$string['tag'] = 'Tag';
$string['username'] = 'Flickr account email';
$string['watermark'] = 'Add watermark to photos';
$string['privacy:metadata:repository_flickr_public'] = 'The Flickr public repository plugin does not store any personal data, but does transmit user data from Moodle to the remote system.';
$string['privacy:metadata:repository_flickr_public:author'] = 'The Flickr public repository content author.';
$string['privacy:metadata:repository_flickr_public:email_address'] = 'The Flickr public repository user email address.';
$string['privacy:metadata:repository_flickr_public:text'] = 'The Flickr public repository user search text.';
$string['privacy:metadata:repository_flickr_public:user_id'] = 'The Flickr public repository user ID.';
