{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/forum_discussion_threaded_posts

    Template to render a list of posts in a threaded format.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Example context (json):
    {
    }
}}

{{#posts}}
    {{< mod_forum/forum_discussion_post }}
        {{$replies}}
            {{! The forumthread class is only added for behat. }}
            <div class="indent forumthread post-replies" data-region="replies-container">
                {{#replies}}
                    {{> mod_forum/forum_discussion_threaded_post }}
                {{/replies}}
            </div>
        {{/replies}}
    {{/ mod_forum/forum_discussion_post }}
{{/posts}}
