{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_user/participantsfilter

    Template for the form containing one or more filter rows.

    Example context (json):
    {
        "filtertypes": [
            {
                "name": "status",
                "title": "Status",
                "values": [
                    {
                        "value": 1,
                        "title": "Active"
                    },
                    {
                        "value": 0,
                        "title": "Suspended"
                    }
                ]
            }
        ]
    }
}}

<div id="core_user-participantsfilter-{{uniqid}}" class="filter-group my-2 p-2 bg-light border-radius border" data-table-region="{{tableregionid}}" data-table-course-id="{{courseid}}" data-filterverb="2">
    <div data-filterregion="filtermatch" class="hidden">
        <label for="core_user-local-participantsfilter-jointype-{{uniqid}}" class="my-0" aria-hidden="true">
            {{#str}}match, core_user{{/str}}
        </label>
        <select class="custom-select" data-filterfield="join" id="core_user-local-participantsfilter-jointype-{{uniqid}}"
                aria-label="{{#str}}filtersetmatchdescription, core_user{{/str}}">
            <option value="0">{{#str}}none{{/str}}</option>
            <option value="1">{{#str}}any{{/str}}</option>
            <option value="2" selected>{{#str}}all{{/str}}</option>
        </select>
        <span aria-hidden="true">{{#str}}matchofthefollowing, core_user{{/str}}</span>
    </div>

    <div data-filterregion="filters">
        {{> core_user/local/participantsfilter/filterrow }}
    </div>

    <div class="d-flex" data-filterregion="actions">
        &nbsp;
        <button type="button" class="btn btn-link text-reset" data-filteraction="add">
            <i class="fa fa-plus"></i><span class="pl-3">{{#str}}addcondition, core_user{{/str}}</span>
        </button>
        <button data-filteraction="reset" type="button" class="btn btn-secondary ml-auto mr-2">{{#str}}clearfilters, core_user{{/str}}</button>
        <button data-filteraction="apply" type="button" class="btn btn-primary">{{#str}}applyfilters, core_user{{/str}}</button>
    </div>

    {{> core_user/local/participantsfilter/filtertypes}}
</div>

{{#js}}
require(['core_user/participantsfilter'], function(ParticipantsFilter) {
    ParticipantsFilter.init('core_user-participantsfilter-{{uniqid}}');
});
{{/js}}
