<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Automatically generated strings for Moodle installer
 *
 * Do not edit this file manually! It contains just a subset of strings
 * needed during the very first steps of installation. This file was
 * generated automatically by export-installer.php (which is part of AMOS
 * {@link http://docs.moodle.org/dev/Languages/AMOS}) using the
 * list of strings defined in /install/stringnames.txt.
 *
 * @package   installer
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

$string['clianswerno'] = 'не';
$string['cliansweryes'] = 'ҳа';
$string['cliincorrectvalueerror'] = 'Хато, нишондиҳандаи нодурусти "{$a->value}" барои "{$a->option}"';
$string['cliincorrectvalueretry'] = 'Нишондиҳандаи нодуруст, илтимос, аз нав кӯшиш карда бинед';
$string['clitypevalue'] = 'Нишондиҳандаро ворид намоед';
$string['clitypevaluedefault'] = 'Нишондиҳандаро ворид намоед, Enter-ро барои истифодаи нишондиҳанда аз рӯи хомӯшӣ ({$a}) зер кунед';
$string['cliunknowoption'] = 'Параметрҳои шинохтанашуда:
  {$a}
Илтимос, аз параметри --help истифода баред';
$string['cliyesnoprompt'] = 'Ҳарфҳои y (яъне Ҳа) ё n (яъне Не)';
$string['environmentrequireinstall'] = 'Ҷойгир карда ба кор андохтан лозим аст';
$string['environmentrequireversion'] = 'Версияи {$a->needed} лозим аст, Шумо версияи {$a->current} -ро истифода бурда истодаед';
