<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Automatically generated strings for Moodle installer
 *
 * Do not edit this file manually! It contains just a subset of strings
 * needed during the very first steps of installation. This file was
 * generated automatically by export-installer.php (which is part of AMOS
 * {@link http://docs.moodle.org/dev/Languages/AMOS}) using the
 * list of strings defined in /install/stringnames.txt.
 *
 * @package   installer
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

$string['admindirname'] = '\'Etimini fakahinohino';
$string['dataroot'] = 'Fakahinohino tata';
$string['dbprefix'] = 'Tepile pulifiki';
$string['dirroot'] = 'Mutolo fakahinohino';
$string['installation'] = 'Fokotu\'u';
$string['memorylimithelp'] = '<p> Koe memolii PHP limiti ki ho\'o seeva \'oku lolotonga seti ki he {$a}.</p>

<p>\'E malava eni ke fakalanga\'i e Muutolo ke \'iai ha ngaahi memolii palopalema \'a mui ange , tautautefito kapau \'oku ma\'u ha ngaahi motiula \'oku malava pea/mo ha kau \'iusa tokolahi.

<p>\'Oku mau faka\'ai\'ai koe keke fakafika ho\'o PHP aki ha limiti ma\'olunga kapau \'e lava, hange koe 40M.
   Koe ngaahi founga kehekehe eni teke lava \'o \'ahi\'ahi\'i ki hono fai \'o \'eni:
<ol>
<li>Kapau teke lava,toe fakatahatah\'i e PHP moe <i>--malava-memolii-limiti</i>.
    \'E hanga he\'e me\'a ko\'eni \'o fakangofua e Muutolo kene seti pee e limiti \'oe memolii \'eia pee.
<li>Kapau \'oku \'iai ha\'o \'ekisesi ki ho\'o php.ini faile, teke lava \'o liliu <b>memolii_limiti</b>
    \'o seti\'i ia ki ha me\'a hangee koe 40M. Kapau \'oku \'ikai ke \'iai ha\'o \'ekisesi, teke kei malava pe keke kole ki ho\'o \'etiminisituleitaa kene fai \'eni ma\'au.
<li>\'i he ni\'ihi \'oe ngaahi seeva PHPtekelava \'o fakatupu ha .ht\'ekisesi faile \'i loto \'ihe fakahinohino Muutolo \'oku \'i loto \'i he laine ko \'eni:
    <p><polokakuouti>php_mahu\'inga memolii_limiti 40M</polokakuouti></p>
    <p>Kaekehe, he\'ikai ke lava \'eni ia \'i he ni\'ihi \'oe ngaahi seeva<b>all</b> PHP ngaahi peesi \'oku ngaue
    (teke lava \'o sio ki he ngaahi fehalaaki \'i ha\'o sio ki he ngaahi peesi) koia ai kuopau keke to\'o e .ht\'ekisesi faile.
</ol>';
$string['phpversion'] = 'vesiniPHP';
$string['phpversionhelp'] = '<p>\'Oku fiema\'u he\'e Muutolo ha vesini PHP \'oku laka hake he 4.1.0.</p>
<p>\'Oku ke lolotonga fakalele a\'e vesini {$a}</p>
<p>\'Oku totonu keke \'apukuleiti e PHP pee hiki ki ha housi moha vesini fo\'ou \'oe PHP!</p>';
$string['wwwroot'] = 'Tu\'asila Uepi';
