<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'repository_filesystem', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package   repository_filesystem
 * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['blockedpath'] = 'Blocked root path';
$string['configplugin'] = 'Configuration for file system repository';
$string['crontask'] = 'Background processing for file system repository';
$string['donotusesysdir'] = 'Don\'t use system directory as repository, including';
$string['filesystem:view'] = 'View file system repository';
$string['information'] = 'These folders are within the <b>{$a}</b> directory.';
$string['invalidpath'] = 'Invalid root path';
$string['path'] = 'Select a subdirectory';
$string['relativefiles'] = 'Allow relative files';
$string['relativefiles_desc'] = 'This allows all files in the repository to be accessible using relative links.';
$string['root'] = 'Root';
$string['nosubdir'] = 'You need to create at least one folder inside the <b>{$a}</b> directory so you can select it here.';
$string['pluginname_help'] = 'Create repository from local directory';
$string['pluginname'] = 'File system';
$string['searchresults'] = 'Search results';
$string['enablecourseinstances'] = 'Allow admins to add a file system repository instance to a course (configurable only by admins)';
$string['enableuserinstances'] = 'Allow admins to add a file system repository instance for personal use (configurable only by admins)';
$string['privacy:metadata'] = 'The File system repository plugin does not store or transmit any personal data.';
