<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Automatically generated strings for Moodle installer
 *
 * Do not edit this file manually! It contains just a subset of strings
 * needed during the very first steps of installation. This file was
 * generated automatically by export-installer.php (which is part of AMOS
 * {@link http://docs.moodle.org/dev/Languages/AMOS}) using the
 * list of strings defined in /install/stringnames.txt.
 *
 * @package   installer
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

$string['admindirname'] = 'ایڈمن ڈائرکٹری';
$string['availablelangs'] = 'دستیاب لینگویج پیک';
$string['chooselanguagehead'] = 'ایک زبان کا انتخاب کریں';
$string['chooselanguagesub'] = 'براہ کرم انسٹالیشن کے لیے ایک زبان کا انتخاب کریں۔ اس زبان کو سائٹ کے لیے ڈیفالٹ زبان کے طور پر بھی استعمال کیا جائے گا، حالانکہ اسے بعد میں تبدیل کیا جا سکتا ہے۔';
$string['clialreadyconfigured'] = 'کنفیگریشن فائل config.php پہلے سے موجود ہے۔ اس سائٹ کے لیے Moodle انسٹال کرنے کے لیے براہ کرم admin/cli/install_database.php استعمال کریں۔';
$string['clialreadyinstalled'] = 'کنفیگریشن فائل config.php پہلے سے موجود ہے۔ اس سائٹ کے لیے Moodle کو اپ گریڈ کرنے کے لیے براہ کرم admin/cli/install_database.php استعمال کریں۔';
$string['cliinstallheader'] = 'موڈل {$a} کمانڈ لائن انسٹالیشن پروگرام';
$string['clitablesexist'] = 'ڈیٹا بیس کی میزیں پہلے سے موجود ہیں؛ CLI کی تنصیب جاری نہیں رہ سکتی۔';
$string['databasehost'] = 'ڈیٹا بیس میزبان';
$string['databasename'] = 'ڈیٹا بیس نام';
$string['databasetypehead'] = 'ڈیٹا بیس ڈرائیور کا انتخاب کریں';
$string['dataroot'] = 'ڈیٹا ڈائریکٹری';
$string['datarootpermission'] = 'ڈیٹا ڈائریکٹریز کی اجازت';
$string['dbprefix'] = 'جدولوں کا سابقہ';
$string['dirroot'] = 'موڈل ڈائرکٹری';
$string['environmenthead'] = 'اپنے ماحول کی جانچ کر رہا ہے۔۔۔';
$string['environmentsub2'] = 'ہر موڈل ریلیز میں کچھ کم از کم پی ایچ پی ورژن کی ضرورت اور متعدد لازمی پی ایچ پی ایکسٹینشنز ہوتے ہیں۔ ہر انسٹال اور اپ گریڈ سے پہلے ماحول کی مکمل جانچ کی جاتی ہے۔ اگر آپ نیا ورژن انسٹال کرنا یا پی ایچ پی ایکسٹینشنز کو فعال کرنا نہیں جانتے ہیں تو براہ کرم سرور ایڈمنسٹریٹر سے رابطہ کریں۔';
$string['errorsinenvironment'] = 'ماحولیات کی جانچ ناکام!';
$string['installation'] = 'تنصیب';
$string['langdownloaderror'] = 'بدقسمتی سے زبان "{$a}" ڈاؤن لوڈ نہیں ہو سکی۔ تنصیب کا عمل انگریزی میں جاری رہے گا۔';
$string['memorylimithelp'] = '<p>آپ کے سرور کے لیے PHP میموری کی حد فی الحال {$a} پر سیٹ ہے۔</p> <p>اس کی وجہ سے Moodle کو بعد میں میموری کے مسائل کا سامنا کرنا پڑ سکتا ہے، خاص طور پر اگر آپ کے پاس بہت سارے ماڈیولز فعال ہیں اور/یا بہت سارے صارف۔ ایسا کرنے کے کئی طریقے ہیں جنہیں آپ آزما سکتے ہیں:</p> <ol> <li>اگر آپ قابل ہیں تو، پی ایچ پی کو <i>--enable-memory-limit</i> کے ساتھ دوبارہ مرتب کریں۔ یہ موڈل کو میموری کی حد خود سیٹ کرنے کی اجازت دے گا۔</li> <li>اگر آپ کو اپنی php.ini فائل تک رسائی حاصل ہے، تو آپ وہاں کی <b>memory_limit</b> سیٹنگ کو 40M جیسی چیز میں تبدیل کر سکتے ہیں۔ اگر آپ کے پاس رسائی نہیں ہے تو آپ اپنے منتظم سے اپنے لیے ایسا کرنے کے لیے کہہ سکتے ہیں۔</li> <li>کچھ پی ایچ پی سرورز پر آپ موڈل ڈائرکٹری میں ایک .htaccess فائل بنا سکتے ہیں جس میں یہ لائن ہے: <blockquote> <div>php_value memory_limit 40M</div></blockquote> <p>تاہم، کچھ سرورز پر یہ <b>تمام</b> پی ایچ پی صفحات کو کام کرنے سے روک دے گا (صفحات کو دیکھتے وقت آپ کو غلطیاں نظر آئیں گی) تاکہ آپ کو .htaccess فائل کو ہٹانا پڑے گا۔</p></li></ol>';
$string['paths'] = 'راستے';
$string['pathserrcreatedataroot'] = 'ڈیٹا ڈائرکٹری ({$a->dataroot}) انسٹالر کے ذریعے نہیں بنائی جا سکتی۔';
$string['pathshead'] = 'راستوں کی تصدیق کریں';
$string['pathsrodataroot'] = 'Dataroot ڈائریکٹری قابل تحریر نہیں ہے۔';
$string['pathsroparentdataroot'] = 'پیرنٹ ڈائرکٹری ({$a->parent}) قابل تحریر نہیں ہے۔ ڈیٹا ڈائرکٹری ({$a->dataroot}) انسٹالر کے ذریعے نہیں بنائی جا سکتی۔';
$string['pathssubadmindir'] = 'بہت کم ویب ہوسٹس /admin کو بطور خاص یو آر ایل استعمال کرتے ہیں تاکہ آپ کنٹرول پینل یا کسی اور چیز تک رسائی حاصل کر سکیں۔ بدقسمتی سے یہ موڈل ایڈمن پیجز کے معیاری مقام سے متصادم ہے۔ آپ اسے اپنی انسٹالیشن میں ایڈمن ڈائرکٹری کا نام بدل کر، اور اس نئے نام کو یہاں رکھ کر ٹھیک کر سکتے ہیں۔ مثال کے طور پر: <em>moodleadmin</em>۔ یہ Moodle میں ایڈمن لنکس کو ٹھیک کر دے گا۔';
$string['pathssubdataroot'] = '<p>ایک ڈائرکٹری جہاں Moodle صارفین کی طرف سے اپ لوڈ کردہ تمام فائل مواد کو اسٹور کرے گا۔</p> <p>یہ ڈائرکٹری ویب سرور صارف (عام طور پر \'www-data\'، \'nobody\'، یا \'کے ذریعے پڑھنے اور لکھنے کے قابل دونوں ہونی چاہیے۔ apache\')۔</p> <p>یہ ویب پر براہ راست قابل رسائی نہیں ہونا چاہیے۔</p> <p>اگر ڈائرکٹری فی الحال موجود نہیں ہے تو انسٹالیشن کا عمل اسے بنانے کی کوشش کرے گا۔</p>';
$string['pathssubdirroot'] = '<p>موڈل کوڈ پر مشتمل ڈائریکٹری کا مکمل راستہ۔</p>';
$string['pathssubwwwroot'] = '<p>مکمل پتہ جہاں Moodle تک رسائی حاصل کی جائے گی یعنی وہ ایڈریس جسے صارفین Moodle تک رسائی کے لیے اپنے براؤزر کے ایڈریس بار میں داخل کریں گے۔</p> <p>متعدد پتوں کا استعمال کرتے ہوئے Moodle تک رسائی ممکن نہیں ہے۔ اگر آپ کی سائٹ ایک سے زیادہ پتوں کے ذریعے قابل رسائی ہے تو پھر سب سے آسان کا انتخاب کریں اور دوسرے پتے میں سے ہر ایک کے لیے مستقل ری ڈائریکٹ سیٹ کریں۔</p> <p>اگر آپ کی سائٹ انٹرنیٹ اور اندرونی نیٹ ورک دونوں سے قابل رسائی ہے (کبھی کبھی ایک انٹرانیٹ کہا جاتا ہے)، پھر یہاں عوامی پتہ استعمال کریں۔</p> <p>اگر موجودہ پتہ درست نہیں ہے، تو براہ کرم اپنے براؤزر کے ایڈریس بار میں URL کو تبدیل کریں اور انسٹالیشن دوبارہ شروع کریں۔</p>';
$string['pathsunsecuredataroot'] = 'Dataroot مقام محفوظ نہیں ہے';
$string['pathswrongadmindir'] = 'ایڈمن ڈائرکٹری موجود نہیں ہے';
$string['phpextension'] = '{$a} پی ایچ پی کی توسیع';
$string['phpversion'] = 'پی ایچ پی ورژن';
$string['phpversionhelp'] = '<p>موڈل کو کم از کم 5.6.5 یا 7.1 کا پی ایچ پی ورژن درکار ہے (7.0.x میں انجن کی کچھ حدود ہیں)۔</p> <p>آپ فی الحال ورژن {$a} چلا رہے ہیں۔</p> <p> آپ کو پی ایچ پی کو اپ گریڈ کرنا ہوگا یا پی ایچ پی کے نئے ورژن کے ساتھ میزبان پر جانا ہوگا۔</p>';
$string['welcomep20'] = 'آپ یہ صفحہ دیکھ رہے ہیں کیونکہ آپ نے اپنے کمپیوٹر میں <strong>{$a->packname} {$a->packversion}</strong> پیکیج کو کامیابی سے انسٹال اور لانچ کیا ہے۔ مبارک ہو!';
$string['welcomep30'] = '<strong>{$a->installername</strong> کی اس ریلیز میں ایک ایسا ماحول بنانے کے لیے ایپلیکیشنز شامل ہیں جس میں <strong>Moodle</strong> کام کرے گا، یعنی:';
$string['welcomep40'] = 'پیکیج میں <strong>Moodle {$a->moodlerelease} ({$a->moodleversion})</strong> بھی شامل ہے۔';
$string['welcomep50'] = 'اس پیکج میں تمام درخواستوں کا استعمال ان کے متعلقہ لائسنسوں کے ذریعے کیا جاتا ہے۔ مکمل <strong>{$a->installername}</strong> پیکیج <a href="https://www.opensource.org/docs/definition_plain.html">اوپن سورس</a> ہے اور اس کے تحت تقسیم کیا گیا ہے۔ <a href="https://www.gnu.org/copyleft/gpl.html">GPL</a> لائسنس۔';
$string['welcomep60'] = 'مندرجہ ذیل صفحات آپ کو اپنے کمپیوٹر پر <strong>Moodle</strong> کو ترتیب دینے اور ترتیب دینے کے لیے کچھ آسان اقدامات کی پیروی کرنے کی رہنمائی کریں گے۔ آپ پہلے سے طے شدہ ترتیبات کو قبول کر سکتے ہیں یا اختیاری طور پر، اپنی ضروریات کے مطابق ان میں ترمیم کر سکتے ہیں۔';
$string['welcomep70'] = '<strong>Moodle</strong> کے سیٹ اپ کے ساتھ جاری رکھنے کے لیے نیچے "اگلا" بٹن پر کلک کریں۔';
$string['wwwroot'] = 'ویب سائٹ کا پتہ';
