.path-admin-tool-health div#healthnoproblemsfound {
    width: 60%;
    margin: auto;
    padding: 1em;
    border: 1px solid black;
    -moz-border-radius: 6px;
}

.path-admin-tool-health dl.healthissues {
    width: 60%;
    margin: auto;
}

.path-admin-tool-health dl.critical dt,
.path-admin-tool-health dl.critical dd {
    background-color: #a71501;
}

.path-admin-tool-health dl.significant dt,
.path-admin-tool-health dl.significant dd {
    background-color: #d36707;
}

.path-admin-tool-health dl.annoyance dt,
.path-admin-tool-health dl.annoyance dd {
    background-color: #dba707;
}

.path-admin-tool-health dl.notice dt,
.path-admin-tool-health dl.notice dd {
    background-color: #e5db36;
}

.path-admin-tool-health dl dt.solution,
.path-admin-tool-health dl dd.solution,
.path-admin-tool-health div#healthnoproblemsfound {
    background-color: #5bb83e;
}

.path-admin-tool-health dl.healthissues dt,
.path-admin-tool-health dl.healthissues dd {
    margin: 0;
    padding: 1em;
    border: 1px solid black;
}

.path-admin-tool-health dl.healthissues dt {
    font-weight: bold;
    border-bottom: 0;
    padding-bottom: 0.5em;
}

.path-admin-tool-health dl.healthissues dd {
    border-top: 0;
    padding-top: 0.5em;
    margin-bottom: 10px;
}

.path-admin-tool-health dl.healthissues dd form {
    margin-top: 0.5em;
    text-align: right;
}

.path-admin-tool-health form#healthformreturn {
    text-align: center;
    margin: 2em;
}

.path-admin-tool-health dd.solution p {
    padding: 0;
    margin: 1em 0;
}

.path-admin-tool-health dd.solution li {
    margin-top: 1em;
}

