{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/quick_search_form

    This template renders the search form within forums.

    Example context (json):
    {
        "action": "https://domain.example/mod/forum/search.php",
        "helpicon": "<a class='btn'><i class='icon fa fa-question-circle'></i></a>",
        "query": "find this post",
        "inputname": "search"
    }
}}
{{#helpicon}}
    {{>core/help_icon}}
{{/helpicon}}
{{< core/search_input }}
{{/ core/search_input }}
