<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Automatically generated strings for Moodle installer
 *
 * Do not edit this file manually! It contains just a subset of strings
 * needed during the very first steps of installation. This file was
 * generated automatically by export-installer.php (which is part of AMOS
 * {@link http://docs.moodle.org/dev/Languages/AMOS}) using the
 * list of strings defined in /install/stringnames.txt.
 *
 * @package   installer
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

$string['cannotcreatedboninstall'] = '<p>Impossible de crear la banca de dadas.</p>
<p>La banca de dadas indicada n\'existeish pas e l\'utilizaire especificat a pas las autorizacions que permeton de crear ua banca de dadas.</p>.
<p>L\'administrator deu site deu repassar la configuracion de la banca de dadas.</p>';
$string['cannotcreatelangdir'] = 'Creacion deu dossièr lang impossible';
$string['cannotcreatetempdir'] = 'Creacion deu dossièr temp impossibla';
$string['cannotdownloadcomponents'] = 'Telecargament deus components impossible';
$string['cannotdownloadzipfile'] = 'Telecargament deu fichèr ZIP impossible';
$string['cannotfindcomponent'] = 'Component introbable';
$string['cannotsavemd5file'] = 'Enregistrament deu fichèr md5 impossible';
$string['cannotsavezipfile'] = 'Enregistrament deu fichèr ZIP impossible';
$string['cannotunzipfile'] = 'Descompression deu fichèr ZIP impossibla';
$string['componentisuptodate'] = 'Lo component qu\'ei a dia';
$string['dmlexceptiononinstall'] = '<p>Ua error de banca de dadas que s\'ei produsida [{$a->errorcode}].<br />{$a->debuginfo}</p>';
$string['downloadedfilecheckfailed'] = 'La verificacion deu fichèr telecargat s\'i ei mauescadut';
$string['invalidmd5'] = 'Lo còde de contraròtle md5 n\'ei pas valid';
$string['missingrequiredfield'] = 'Un camp obligatòri n\'ei pas completat';
$string['remotedownloaderror'] = '<p>Lo telecargament deu component suu vòste servidor s\'i ei mauescadut. Verificatz los reglatges de proxy. L\'extension cURL de PHP qu\'ei bravament recomandada.</p>
<p>Que\'vs cau telecargar manuaument lo fichèr <a href="{$a->url}">{$a->url}</a>, lo copiar suu vòste servidor a l\'emplaçament « {$a->dest} » e lo descompressar a aqueste endret.</p>';
$string['wrongdestpath'] = 'Camin de destinacion incorrècte';
$string['wrongsourcebase'] = 'Adreça URL de basa de la hont incorrècta';
$string['wrongzipfilename'] = 'Nom de fichèr ZIP incorrècte';
