<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Course module instances list_viewed event.
 *
 * This class has been deprecated, please use \core\event\course_module_instance_list_viewed.
 *
 * @package    core
 * @copyright  2013 Frédéric Massart
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace core\event;
defined('MOODLE_INTERNAL') || die();

debugging('core\\event\\course_module_instances_list_viewed has been deperecated. Please use
        core\\event\\course_module_instance_list_viewed instead', DEBUG_DEVELOPER);

/**
 * This class has been deprecated, please use \core\event\course_module_instance_list_viewed.
 *
 * @deprecated Since Moodle 2.7
 * @package    core
 * @since      Moodle 2.6
 * @copyright  2013 Frédéric Massart
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
abstract class course_module_instances_list_viewed extends course_module_instance_list_viewed {
    /**
     * This event has been deprected.
     *
     * @return boolean
     */
    public static function is_deprecated() {
        return true;
    }
}
