{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!

    @template theme_boost/navbar

    This template renders the top navbar.

    Example context (json):
    {
        "output": {
            "should_display_navbar_logo": true,
            "get_compact_logo_url": "http://placekitten.com/50/50",
            "custom_menu": "<li>..</li>",
            "page_heading_menu": "<li>..</li>",
            "search_box": "<div><input type='text'></div>",
            "navbar_plugin_output": "<div class='dropdown'>Messaging</div>",
            "user_menu": "<div class='dropdown'>John Doe</div>"
        },
        "config": {
            "wwwroot": "#"
        },
        "sitename": "Moodle Site"
    }
}}
<nav class="fixed-top navbar navbar-light bg-white navbar-expand moodle-has-zindex" aria-label="{{#str}}sitemenubar, admin{{/str}}">

        <div data-region="drawer-toggle" class="d-inline-block mr-3">
            <button aria-expanded="{{#navdraweropen}}true{{/navdraweropen}}{{^navdraweropen}}false{{/navdraweropen}}" aria-controls="nav-drawer" type="button" class="btn nav-link float-sm-left mr-1 btn-light bg-gray" data-action="toggle-drawer" data-side="left" data-preference="drawer-open-nav">{{#pix}}i/menubars{{/pix}}<span class="sr-only">{{#str}}sidepanel, core{{/str}}</span></button>
        </div>

        <a href="{{{ config.wwwroot }}}" class="navbar-brand aabtn {{# output.should_display_navbar_logo }}has-logo{{/ output.should_display_navbar_logo }}
            {{^ output.should_display_navbar_logo }}
                d-none d-sm-inline
            {{/ output.should_display_navbar_logo }}
                ">
            {{# output.should_display_navbar_logo }}
                <span class="logo d-none d-sm-inline">
                    <img src="{{output.get_compact_logo_url}}" alt="{{sitename}}">
                </span>
            {{/ output.should_display_navbar_logo }}
            <span class="site-name d-none d-md-inline">{{{ sitename }}}</span>
        </a>

        <ul class="navbar-nav d-none d-md-flex">
            <!-- custom_menu -->
            {{{ output.custom_menu }}}
            <!-- page_heading_menu -->
            {{{ output.page_heading_menu }}}
        </ul>
        <div class="ml-auto">
            {{{ output.search_box }}}
        </div>
        <ul class="nav navbar-nav usernav">
            <!-- navbar_plugin_output -->
            <li class="nav-item">
                {{{ output.navbar_plugin_output }}}
            </li>
            <!-- user_menu -->
            <li class="nav-item align-items-center pl-2">
                {{{ output.user_menu }}}
            </li>
        </ul>
        <!-- search_box -->
</nav>
