{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_course/course_search_form

    This template renders the form for course search.

    Example context (json):
    {
        "searchurl": "https://moodlesite/course/search.php",
        "id": "coursesearch",
        "inputid": "coursesearchbox",
        "inputsize": 30,
        "value": "certificate",
        "name" : "q",
        "helpicon": [{
            "heading": "Search courses",
            "text": "You can search for multiple words at once and can refine your search as follows:"
        }]
    }
}}
<form action="{{searchurl}}" id="{{id}}" method="get" class="form-inline">
    <fieldset class="coursesearchbox invisiblefieldset">
        <label for="{{inputid}}">{{#str}}searchcourses{{/str}}</label>
        <input id="{{inputid}}" name="q" type="text" size="{{inputsize}}" value="{{value}}" class="form-control mb-1 mb-sm-0">
        <input name="areaids" type="hidden" value="{{areaids}}">
        <button class="btn btn-secondary" type="submit">{{#str}}go{{/str}}</button>
        {{#helpicon}}
            {{>core/help_icon}}
        {{/helpicon}}
    </fieldset>
</form>
