<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Automatically generated strings for Moodle installer
 *
 * Do not edit this file manually! It contains just a subset of strings
 * needed during the very first steps of installation. This file was
 * generated automatically by export-installer.php (which is part of AMOS
 * {@link http://docs.moodle.org/dev/Languages/AMOS}) using the
 * list of strings defined in /install/stringnames.txt.
 *
 * @package   installer
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

$string['cannotcreatelangdir'] = 'Չի հաջողվել ստեղծել lang դիրեկտորիա';
$string['cannotcreatetempdir'] = 'Չի հաջողվել ստեղծել temp դիրեկտորիա';
$string['cannotdownloadcomponents'] = 'Չի կարող ներլցնել բաղադրիչներ։';
$string['cannotdownloadzipfile'] = 'Չի կարող ներլցնել ZIP ֆայլ';
$string['cannotfindcomponent'] = 'Չի հաջողվել գտնել բաղադրիչը';
$string['cannotsavemd5file'] = 'Չի կարող պահպանել md5 ֆայլ';
$string['cannotsavezipfile'] = 'Չի կարող պահպանել ZIP ֆայլ';
$string['cannotunzipfile'] = 'Չի կարող unzip անել ֆայլը';
$string['componentisuptodate'] = 'Բաղադրիչը ժամանակակից է';
$string['downloadedfilecheckfailed'] = 'Ներլցված ֆայլի ստուգումը ձախողվել է';
$string['invalidmd5'] = 'Անվավեր md5';
$string['missingrequiredfield'] = 'Պարտադիր դաշտը լրացված չէ';
$string['remotedownloaderror'] = 'Ձեր սերվեր բաղադրիչների ներլցնումը ձախողվել է, խնդրվում է ստուգել միջնորդ սերվերի (proxy) կարգաբերումները, խորհուրդ է տրվում PHP cURL ընդլայնումը։<br /><br />Ձեզ անհրաժեշտ է ձեռքով ներլցնել <a href="{$a->url}">{$a->url}</a> ֆայլը, այն պատճենել սերվերում՝ "{$a->dest}" և unzip անել այնտեղ։';
$string['wrongdestpath'] = 'Նպատակակետի սխալ ուղի';
$string['wrongsourcebase'] = 'Սխալ URL հասցե';
$string['wrongzipfilename'] = 'ZIP ֆայլի սխալ անվանում';
