.path-admin-mnet-service-enrol .singlebutton {
    text-align: center;
}
.path-admin-mnet-service-enrol table.remotehosts,
.path-admin-mnet-service-enrol table.otherenrolledusers,
.path-admin-mnet-service-enrol table.remotecourses {
    margin: 0 auto 1em auto;
}
.path-admin-mnet-service-enrol table.remotecourses th.categoryname {
    text-align: left;
    background-color: #f6f6f6;
}
.path-admin-mnet-service-enrol table.remotecourses td.c1 {
    font-weight: bold;
}
.path-admin-mnet-service-enrol table.remotecourses th.categoryname img {
    margin-right: 1em;
}
.path-admin-mnet-service-enrol .collapsibleregioncaption {
    font-size: 110%;
    font-weight: bold;
    text-align: center;
}
.path-admin-mnet-service-enrol .collapsibleregioninner {
    border: 1px solid #ddd;
    padding: 1em;
}
.path-admin-mnet-service-enrol .collapsibleregion.remotecourse.summary {
    margin: 0 10em;
}
.path-admin-mnet-service-enrol .roleassigntable {
    margin: 1em auto;
}
