YUI.add("moodle-core-notification-alert",function(i,t){var e;M.core=M.core||{},i.extend(e=function(t){t.closeButton=!1,e.superclass.constructor.apply(this,[t])},M.core.notification.info,{_closeEvents:null,initializer:function(){var t,e;this._closeEvents=[],this.publish("complete"),t=i.Node.create('<input type="button" class="btn btn-primary" id="id_yuialertconfirm-'+this.get("COUNT")+'"value="'+this.get("yesLabel")+'" />'),e=i.Node.create('<div class="confirmation-dialogue"></div>').append(i.Node.create('<div class="confirmation-message">'+this.get("message")+"</div>")).append(i.Node.create('<div class="confirmation-buttons text-xs-right"></div>').append(t)),this.get("notificationBase").addClass("moodle-dialogue-confirm"),this.setStdModContent(i.WidgetStdMod.BODY,e,i.WidgetStdMod.REPLACE),this.setStdModContent(i.WidgetStdMod.HEADER,'<h5 id="moodle-dialogue-'+this.get("COUNT")+'-wrap-header-text">'+this.get("title")+"</h5>",i.WidgetStdMod.REPLACE),this._closeEvents.push(i.on("key",this.submit,window,"down:13",this),t.on("click",this.submit,this)),(e=this.get("boundingBox").one(".closebutton"))&&this._closeEvents.push(e.on("click",this.submit,this))},submit:function(){new i.EventHandle(this._closeEvents).detach(),this.fire("complete"),this.hide(),this.destroy()}},{NAME:"Moodle alert",CSS_PREFIX:"moodle-dialogue",ATTRS:{title:{validator:i.Lang.isString,value:"Alert"},message:{validator:i.Lang.isString,value:"Confirm"},yesLabel:{validator:i.Lang.isString,setter:function(t){return t=t||"OK"},value:"OK"}}}),M.core.alert=e},"@VERSION@",{requires:["moodle-core-notification-dialogue"]});