<?php

namespace IMSGlobal\LTI\OAuth;

/**
 * Class to represent an %OAuth Exception
 *
 * @copyright  Andy Smith
 * @version 2008-08-04
 * @license https://opensource.org/licenses/MIT The MIT License
 */
class OAuthException extends \Exception {
  // pass
}
