<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Automatically generated strings for Moodle installer
 *
 * Do not edit this file manually! It contains just a subset of strings
 * needed during the very first steps of installation. This file was
 * generated automatically by export-installer.php (which is part of AMOS
 * {@link http://docs.moodle.org/dev/Languages/AMOS}) using the
 * list of strings defined in /install/stringnames.txt.
 *
 * @package   installer
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

$string['cannotcreatelangdir'] = 'மொழி அடைவை உருவாக்க முடியவில்லை';
$string['cannotcreatetempdir'] = 'தற்காலிக அடைவை உருவாக்க முடியவில்லை';
$string['cannotdownloadcomponents'] = 'கூறுகளைப் பதிவிறக்க முடியவில்லை.';
$string['cannotdownloadzipfile'] = 'ZIP கோப்பைப் பதிவிறக்க முடியவில்லை.';
$string['cannotfindcomponent'] = 'கூறைக் காணவில்லை.';
$string['cannotsavemd5file'] = 'md5 கோப்பைச் சேமிக்க முடியவில்லை.';
$string['cannotsavezipfile'] = 'ZIP கோப்பைச் சேமிக்க முடியவில்லை.';
$string['cannotunzipfile'] = 'கோப்பை unzip செய்ய முடியவில்லை.';
$string['componentisuptodate'] = 'பகுதி ஆனது புத்தாக்கம்செய்யப்பட்டுள்ளது.';
$string['downloadedfilecheckfailed'] = 'பதிவிறக்கப்பட்ட கோப்பு சோதனயில் தேறவில்லை.';
$string['invalidmd5'] = 'செல்லுபடியற்ற md5';
$string['missingrequiredfield'] = 'சில தேவையான புலங்கள் காணப்படவில்லை.';
$string['wrongdestpath'] = 'பிழையான இலக்குப் பாதை';
$string['wrongsourcebase'] = 'பிழையான மூல URL அடிப்படை.';
$string['wrongzipfilename'] = 'பிழையான ZIP கோப்புப் பெயர்.';
