<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * @package assignment_upload
 * @copyright 2011 onwards Aparup Banerjee (nebgor)
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

/**
 * A dummy class for upload conversions.
 * The upload subplugin doesn't really need anything converted.
 * See the moodle1_assignment_subplugin_handler class.
 */
class moodle1_mod_assignment_upload_subplugin_handler extends moodle1_assignment_subplugin_handler {
    /**
     * Override the append_subplugin_data($data) method here for plugin specific processing.
     */
}
