{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_grades/edit_tree

    Edit tree template.

    Context variables required for this template:
    * actionurl - string - Form action URL.
    * sesskey - string - The session key.
    * notification - object - Context data for the notification.
    * showsave - boolean - Whether to show the save changes button.
    * showbulkmove - boolean - Whether to show the bulk move select menu.
    * table - string - HTML content of the grade items table.
    * bulkmoveoptions - array - Key-value pair array for the options of the the bulk move select menu element.

    Example context (json):
    {
        "actionurl": "#",
        "sesskey": "fakesesskey",
        "notification": "",
        "table": "<table class='generaltable simple setup-grades' id='grade_edit_tree_table'><thead> <tr><th>Name</th><th>Weights</th><th>Max grade</th><th>Actions</th> </tr></thead><tbody></tbody></table>",
        "showsave": "true",
        "showbulkmove": "true",
        "bulkmoveoptions": "{'value' : '1', 'name' : 'Test grade category'}",
        "value": "1",
        "name": "Test grade category"
    }
}}

<form id="gradetreeform" method="post" action="{{actionurl}}">
    <div>
        <input type="hidden" name="sesskey" value="{{sesskey}}">
        {{#notification}}
            {{>core/notification_info}}
        {{/notification}}
        {{{table}}}
        <div id="gradetreesubmit">
            {{#showsave}}
                <input class="advanced btn btn-primary" type="submit" value="{{#cleanstr}}savechanges{{/cleanstr}}">
            {{/showsave}}
            {{#showbulkmove}}
                <div class="form-inline mt-3">
                    <input type="hidden" name="bulkmove" value="0" id="bulkmoveinput">
                    <label for="menumoveafter">{{#str}}moveselectedto, grades{{/str}}</label>
                    <select name="moveafter" id="menumoveafter" class="ignoredirty singleselect custom-select form-control"
                            data-action="toggle" data-toggle="action" data-togglegroup="category" disabled>
                        {{#bulkmoveoptions}}
                            <option value="{{value}}">{{{name}}}</option>
                        {{/bulkmoveoptions}}
                    </select>
                </div>
            {{/showbulkmove}}
        </div>
    </div>
</form>
