<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Automatically generated strings for Moodle installer
 *
 * Do not edit this file manually! It contains just a subset of strings
 * needed during the very first steps of installation. This file was
 * generated automatically by export-installer.php (which is part of AMOS
 * {@link http://docs.moodle.org/dev/Languages/AMOS}) using the
 * list of strings defined in /install/stringnames.txt.
 *
 * @package   installer
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

$string['admindirname'] = 'व्यवस्थापकाची डिरेक्टरी';
$string['availablelangs'] = 'उपलब्ध भाषा संच';
$string['chooselanguagehead'] = 'भाषा निवडा';
$string['chooselanguagesub'] = 'क्रुपया फक्त इंस्टॉलेशनसाठी भाषा निवडा. तुम्ही नंतरच्या पानावरती साइट आणि युजरसाठी भाषा निवडू शकाल';
$string['dataroot'] = 'माहीतीची डिरेक्टरी';
$string['dbprefix'] = 'टेबलचे आद्याक्षर';
$string['dirroot'] = 'मूडलची डिरेक्टरी';
$string['environmenthead'] = 'तुमचे पर्यावरण तपासत आहे';
$string['installation'] = 'इंस्टॉलेशन';
$string['phpversion'] = 'PHP ची प्रत';
$string['welcomep60'] = 'खालील पानामध्ये तुम्हाला तुमच्या संगणकावरती मूडल सेट व कॉनफिगर करण्यासाठी काही सोप्या पद्धती दिलेल्या आहेत. तुम्ही डिफॉल्ट सेटींग ठेवू शकता किंवा  तुम्हाला हव्या त्या सुधारणा करू शकता.';
$string['welcomep70'] = 'मूडलचा सेट अप चालू ठेवण्यासाठी \'पूढे\' या बटनवरती क्लिक करा.';
$string['wwwroot'] = 'वेबचा पत्ता';
