{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/local/grades/local/grader/grade_placeholder

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * none

    Example context (json):
    {
    }
}}
<div class="d-flex flex-column mb-3">
    <div class="d-flex align-items-center mb-2" style="height: 36px">
        <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
        <div class="bg-pulse-grey ml-auto mr-2" style="height: 16px; width: 16px"></div>
    </div>
    <div class="d-flex flex-column">
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div>
            <div class="bg-pulse-grey mb-2" style="height: 22px; width: 150px"></div>
            <div class="bg-pulse-grey" style="height: 36px; width: 100%"></div>
        </div>
    </div>
</div>
<div class="d-flex flex-column mb-3">
    <div class="d-flex align-items-center mb-2" style="height: 36px">
        <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
        <div class="bg-pulse-grey ml-auto mr-2" style="height: 16px; width: 16px"></div>
    </div>
    <div class="d-flex flex-column">
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div>
            <div class="bg-pulse-grey mb-2" style="height: 22px; width: 150px"></div>
            <div class="bg-pulse-grey" style="height: 36px; width: 100%"></div>
        </div>
    </div>
</div>
<div class="d-flex flex-column mb-3">
    <div class="d-flex align-items-center mb-2" style="height: 36px">
        <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
        <div class="bg-pulse-grey ml-auto mr-2" style="height: 16px; width: 16px"></div>
    </div>
    <div class="d-flex flex-column">
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div>
            <div class="bg-pulse-grey mb-2" style="height: 22px; width: 150px"></div>
            <div class="bg-pulse-grey" style="height: 36px; width: 100%"></div>
        </div>
    </div>
</div>
<div class="d-flex flex-column mb-3">
    <div class="d-flex align-items-center mb-2" style="height: 36px">
        <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
        <div class="bg-pulse-grey ml-auto mr-2" style="height: 16px; width: 16px"></div>
    </div>
    <div class="d-flex flex-column">
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div>
            <div class="bg-pulse-grey mb-2" style="height: 22px; width: 150px"></div>
            <div class="bg-pulse-grey" style="height: 36px; width: 100%"></div>
        </div>
    </div>
</div>
<div class="d-flex flex-column mb-3">
    <div class="d-flex align-items-center mb-2" style="height: 36px">
        <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
        <div class="bg-pulse-grey ml-auto mr-2" style="height: 16px; width: 16px"></div>
    </div>
    <div class="d-flex flex-column">
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div>
            <div class="bg-pulse-grey mb-2" style="height: 22px; width: 150px"></div>
            <div class="bg-pulse-grey" style="height: 36px; width: 100%"></div>
        </div>
    </div>
</div>
<div class="d-flex flex-column mb-3">
    <div class="d-flex align-items-center mb-2" style="height: 36px">
        <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
        <div class="bg-pulse-grey ml-auto mr-2" style="height: 16px; width: 16px"></div>
    </div>
    <div class="d-flex flex-column">
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div>
            <div class="bg-pulse-grey mb-2" style="height: 22px; width: 150px"></div>
            <div class="bg-pulse-grey" style="height: 36px; width: 100%"></div>
        </div>
    </div>
</div>
<div class="d-flex flex-column mb-3">
    <div class="d-flex align-items-center mb-2" style="height: 36px">
        <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
        <div class="bg-pulse-grey ml-auto mr-2" style="height: 16px; width: 16px"></div>
    </div>
    <div class="d-flex flex-column">
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div class="d-flex align-items-center mb-2" style="height: 22px">
            <div class="bg-pulse-grey rounded-circle mr-1" style="height: 16px; width: 16px"></div>
            <div class="bg-pulse-grey" style="height: 19px; width: 100px"></div>
            <div class="bg-pulse-grey ml-auto" style="height: 19px; width: 50px"></div>
        </div>
        <div>
            <div class="bg-pulse-grey mb-2" style="height: 22px; width: 150px"></div>
            <div class="bg-pulse-grey" style="height: 36px; width: 100%"></div>
        </div>
    </div>
</div>