{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_assign/grading_navigation_user_info

    Initial template for the user info panel - until a user is selected
    a loading spinner will be displayed.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * None

    Context variables required for this template:
    * None

    Example context (json):
    { }
}}
{{> mod_assign/loading }}
{{#js}}
require(['mod_assign/grading_navigation_user_info'], function(UserInfo) {
    new UserInfo('[data-region="user-info"]');
});
{{/js}}
